//
//  KeychainAccess.m
//  TechTrainEval
//
//  Created by timothy perfitt on Wed Nov 12 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

#import "KeychainAccess.h"
#include <CoreFoundation/CoreFoundation.h>
#include <Security/Security.h>
#include <CoreServices/CoreServices.h>

@implementation KeychainAccess



+ (NSString *)getPasswordInKeychainWithServer:(NSString *)serverName
									  account:(NSString *)accountName
										 path:(NSString *)path
									 	 port:(NSString *)port
									   useSSL:(BOOL)ssl
									  useSASL:(BOOL)sasl
{
    OSStatus status;
    void *passwordData = nil;
    SecKeychainItemRef itemRef = nil;
    UInt32 passwordLength = nil;
	NSString *returnValue = @"";

	status = SecKeychainFindInternetPassword (   
				NULL,																			// CFTypeRef keychainOrArray
				[serverName length],															// UInt32 serverNameLength
				[serverName cString],															// const char *serverName
				0,																				// UInt32 securityDomainLength
				NULL,																			// const char *securityDomain
				[accountName length],															// UInt32 accountNameLength
				[accountName cString],															// const char *accountName
				[path length],																	// UInt32 pathLength
				[path cString],																	// const char *path
				(short)[port intValue],															// UInt16 port
				(ssl == YES) ? kSecProtocolTypeLDAPS  : kSecProtocolTypeLDAP,					// SecProtocolType protocol
				(sasl == YES) ? 'sasl' : 'simp',												// SecAuthenticationType authenticationType
				&passwordLength,																// UInt32 *passwordLength
				&passwordData,																	// void **passwordData
				&itemRef  );																	// SecKeychainItemRef *itemRef

	if(status==noErr) returnValue = [NSString stringWithCString:passwordData length:passwordLength];

	if(passwordData) (void)SecKeychainItemFreeContent(NULL, passwordData);

	return returnValue;
}

+ (BOOL)storePasswordInKeychainForServer:(NSString *)serverName
								  account:(NSString *)accountName
								 password:(NSString *)password
									 path:(NSString *)path
									 port:(NSString *)port
								   useSSL:(BOOL)ssl
								  useSASL:(BOOL)sasl
{
    NSLog(@"storePasswordInKeychainForServer");
    OSStatus searchStatus,modifyStatus,addStatus;
    BOOL returnVal=NO;
    void *passwordData = nil;
    SecKeychainItemRef itemRef = nil;
    UInt32 passwordLength = nil;

	searchStatus = SecKeychainFindInternetPassword (   
				NULL,																			// CFTypeRef keychainOrArray
				[serverName length],															// UInt32 serverNameLength
				[serverName cString],															// const char *serverName
				0,																				// UInt32 securityDomainLength
				NULL,																			// const char *securityDomain
				[accountName length],															// UInt32 accountNameLength
				[accountName cString],															// const char *accountName
				[path length],																	// UInt32 pathLength
				[path cString],																	// const char *path
				(short)[port intValue],															// UInt16 port
				(ssl == YES) ? kSecProtocolTypeLDAPS  : kSecProtocolTypeLDAP,					// SecProtocolType protocol
				(sasl == YES) ? 'sasl' : 'simp',												// SecAuthenticationType authenticationType
				&passwordLength,																// UInt32 *passwordLength
				&passwordData,																	// void **passwordData
				&itemRef  );																	// SecKeychainItemRef *itemRef

    if (searchStatus==noErr) {
		modifyStatus = SecKeychainItemModifyAttributesAndData (itemRef, NULL, [password length], [password cString]);													// pointer to password data
		if (modifyStatus==noErr)
			returnVal=YES;
	} else {
		if (searchStatus==errSecItemNotFound) {
			addStatus = SecKeychainAddInternetPassword (   
							NULL,																// CFTypeRef keychainOrArray
							[serverName length],												// UInt32 serverNameLength
							[serverName cString],												// const char *serverName
							0,																	// UInt32 securityDomainLength
							NULL,																// const char *securityDomain
							[accountName length],												// UInt32 accountNameLength
							[accountName cString],												// const char *accountName
							[path length],														// UInt32 pathLength
							[path cString],														// const char *path
							(short)[port intValue],												// UInt16 port
							(ssl == YES) ? kSecProtocolTypeLDAPS  : kSecProtocolTypeLDAP,		// SecProtocolType protocol
							(sasl == YES) ? 'sasl' : 'simp',									// SecAuthenticationType authenticationType
							[password length],													// UInt32 *passwordLength
							[password cString],													// void **passwordData
							&itemRef  );														// SecKeychainItemRef *itemRef
							
			if (addStatus==noErr)
				returnVal=YES;
        } // end if
    } // end if

	if(passwordData) (void)SecKeychainItemFreeContent(NULL, passwordData);
	
    return returnVal;
}

@end
