//
//  LDAPAttribute.m
//  LDAPManager
//
//  Created by Steve Burke on Sat Apr 17 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

#import "LDAPAttribute.h"


@implementation LDAPAttribute

- (id)init
{
    [super init];
    children = [[NSMutableArray alloc] init];
	parent = nil;
	isValue = NO;
    return self;
}

// Accessor methods for the string
- (void)setItemName:(NSString *)s
{
    [s retain];
    [itemName autorelease];
    itemName = s;
}
- (NSString *)itemName
{
    return itemName;
}

// Accessor methods for the children
- (LDAPAttribute *)addChildNamed:(NSString *)s
{
    LDAPAttribute *temp = [[[LDAPAttribute alloc] init] autorelease];
    [temp setItemName:s];
    [temp setParent:self];
    [children addObject:temp];
    return temp;
}
- (LDAPAttribute *)childAtIndex:(int)i
{
    return [children objectAtIndex:i];
}
- (int)childrenCount
{
    return [children count];
}

// Accessor methods for the parent
- (void)setParent:(LDAPAttribute *)a
{
	parent = a;
}
- (LDAPAttribute *)getParent
{
	return parent;
}

// Accessors for the isValue
- (void)setIsValue:(BOOL)b
{
	isValue = b;
}
- (BOOL)isValue
{
	return isValue;
}

// Other methods
- (BOOL)expandable
{
    return ([children count] > 1);
}
- (NSArray *)getAllDNs
{
	NSMutableArray *temp = [NSMutableArray arrayWithCapacity:0];
	int x;
	for(x=0;[self childrenCount] > x; x++) {
		if([[[self childAtIndex:x] itemName] isEqual:@"dn"])
			[temp addObject:[[[self childAtIndex:x] childAtIndex:0] itemName]];
	} // end for
	return [temp sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)];
}
- (NSString *)allChildrenNamesAsString
{
    if ([self childrenCount] == 1) return [[self childAtIndex:0] itemName];
    else {
   
	NSString *temp = [NSString stringWithFormat:@"( %d values: ", [self childrenCount]];
	int x;
	for(x=0; [self childrenCount] > (x+1); x++) {
		temp = [NSString stringWithFormat:@"%@\'%@\', ", temp, [[self childAtIndex:x] itemName]];
	} // end for
	temp = [NSString stringWithFormat:@"%@\'%@\' )", temp, [[self childAtIndex:x] itemName]];
	return temp;
    }
}

- (NSDictionary *)childrenAttributes
{
    NSMutableDictionary *currentAttributes = [NSMutableDictionary dictionary];
    int i,j;
    NSMutableArray *existingValues;
    for(i=0; i<[self childrenCount]; i++) {
	for (j=0; j<[[self childAtIndex:i] childrenCount];j++) {
	    if (![[[self childAtIndex:i] itemName] isEqualToString:@"dn"]) {
		NSString *tempKey=[[self childAtIndex:i] itemName];
		NSString *tempObject=[[[self childAtIndex:i] childAtIndex:j] itemName];
		if (existingValues=[currentAttributes objectForKey:tempKey]) [existingValues addObject:tempObject];
		else [currentAttributes setObject:[NSMutableArray arrayWithObject:tempObject] forKey:tempKey];
	    }
	}
    } // end for
    return [NSDictionary dictionaryWithDictionary:currentAttributes];
}
- (BOOL)isNameMultiLined
{
	if([[[self itemName] componentsSeparatedByString:@"\n"] count] > 1) {
		return YES;
	}
	return NO;
}
- (void)dealloc
{
    [children autorelease];
    [itemName autorelease];
    [super dealloc];
}

@end
