//
//  LDAPQuery.m
//  wormbook
//
//  Created by Timothy Perfitt on Sun Feb 29 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//
//#define DEBUG2 1
//#define DEBUG 1
#import "LDAPQuery.h"

@implementation LDAPQuery

#pragma mark Setup Connection
-(LDAPQuery *)initWithHostURL:(NSString *)hurl port:(int)vPort authType:(int)at bindUsername:(NSString *)bun bindPassword:(NSString *)bpw dn:(NSString *)newDN saslUsername:(NSString *)sun overSSL:(BOOL)ussl{
    
    
    self=[super init];
    authType=at;
    [self setHostURL:hurl];
    [self setBaseDN:newDN];
    [self setBindUsername:bun];
    [self setBindPassword:bpw];
    [self setSASLUsername:sun];
    port=vPort;
    useSSL=ussl;
   // if([self bind]) returnVal=self;
    searchTimers=[[NSMutableDictionary dictionary] retain];
    return self;
    
}
#pragma mark Async Searching
-(BOOL)searchWithFilter:(NSString *)filter forAttributes:(NSArray *)ldapAttributeNames scope:(int)scope searchDN:(NSString *)searchDN target:(id)target selector:(SEL)aSelector {

    int i,size;
    char *tempattr;  
    int msgid;
    int err;

    if (ldapAttributeNames) 
	size=[ldapAttributeNames count];
    else 
	size=0;
    char **attrs;
    attrs=malloc(size);
    //    struct berval **photos;
    //    struct berval photo_data;
    
    if (ldapAttributeNames) {
	for (i=0;i<[ldapAttributeNames count];i++) {
	    tempattr=(char *)[[ldapAttributeNames objectAtIndex:i] UTF8String];
	    attrs[i]=tempattr;
	} // end for
	attrs[i]=NULL;
    } // end if
    else attrs=NULL;
    err=ldap_search_ext(ld, [searchDN UTF8String], scope, [filter UTF8String], attrs, 0,NULL,NULL,NULL,LDAP_NO_LIMIT,&msgid);
    if (err!=LDAP_SUCCESS) {
	NSRunAlertPanel(@"Error!",@"error.  Make sure the dns name is correct, and that the server is accepting connections.", @"OK", nil,nil);
	[self setLastError:[NSString stringWithFormat:@"%s",ldap_err2string(err)]];
	return NO;
    }
    else {
	NSDictionary *userInfoDict=[NSDictionary dictionaryWithObjectsAndKeys:target,@"sender",[NSValue valueWithPointer:aSelector],@"selector",[NSNumber numberWithInt:msgid],@"msgid",nil];
	searchTimer=[NSTimer scheduledTimerWithTimeInterval:1/100 target:self selector:@selector(checkSearchStatus:) userInfo:userInfoDict repeats:YES];
	[searchTimers setObject:searchTimer forKey:[NSString stringWithFormat:@"%i",msgid]];
	//NSLog(@"firing search timer with msgid %i",msgid);
    }
    free (attrs);
    [[NSNotificationCenter defaultCenter] postNotificationName:@"SearchStarted" object:self userInfo:[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:msgid],@"msgid",[NSString stringWithFormat:@"Searching %@ for %@",searchDN,scope==LDAP_SCOPE_ONELEVEL?@"Children":@"Attributes"],@"description",nil]];
    
    return YES;
}

-(void)checkSearchStatus:(NSTimer *)timer{
    int i;
    char *dn;
    char *matcheddn = NULL;
    char *text = NULL;
    char **refs = NULL;
    LDAPControl **ctrls = NULL;
    struct berval **bvals;
    int rc;
    LDAPMessage *resultmessage;
    char *resultstring;
    BerElement *berptr;
    int res;
    LDAPMessage *result;
    struct timeval timeout;
    timeout.tv_sec=0;
    timeout.tv_usec=0;
    id notifyObject;
    int msgid=[[[timer userInfo] objectForKey:@"msgid"] intValue];
    NSMutableDictionary *person;
    res=ldap_result( ld, msgid, LDAP_MSG_ONE, &timeout, &result);

    switch (res) {
	case 0:

	    break;
	case LDAP_RES_SEARCH_ENTRY:
	    notifyObject=[[timer userInfo] objectForKey:@"sender"];
	    SEL notifySelector=[[[timer userInfo] objectForKey:@"selector"] pointerValue];
	    
	    person = [NSMutableDictionary dictionary];
//	    [person setObject:@"" forKey:@"attributes"];
	    #ifdef DEBUG 
		NSLog(@"ldap_count_messages is %i",ldap_count_messages( ld, result ));
	    #endif
	    for (resultmessage = ldap_first_message( ld, result ); resultmessage != nil; resultmessage = ldap_next_message( ld, resultmessage )) {
		dn=ldap_get_dn( ld, resultmessage );
	    
	    
		rc = ldap_parse_result( ld, resultmessage,&res, &matcheddn, &text, &refs, &ctrls, 0 );
	    
		ber_memfree( matcheddn );
		NSMutableString *newPerson=[NSMutableString stringWithUTF8String:dn];
		[newPerson replaceOccurrencesOfString:@", " withString:@"," options:NSLiteralSearch range:NSMakeRange(0, [newPerson length])];
		
		NSArray *dnArray=[newPerson componentsSeparatedByString:@","];
		[person setObject:newPerson forKey:@"dn"];
		NSMutableDictionary  *personAttributes=[NSMutableDictionary dictionary];
		for (resultstring = ldap_first_attribute( ld, resultmessage, &berptr );resultstring != NULL; resultstring = ldap_next_attribute( ld, resultmessage, berptr ) ){
		    #ifdef DEBUG2
			NSLog(@"result string is %s",resultstring);
		    #endif
		    if (( bvals = ldap_get_values_len( ld, resultmessage, resultstring )) != NULL ) { 
			for ( i = 0; bvals[i] != NULL; i++ ) {
			    NSData *childData=[NSData dataWithBytes:bvals[i]->bv_val length:bvals[i]->bv_len];
			    NSString *childString=[[[NSString alloc] initWithData:childData encoding:[NSString defaultCStringEncoding]] autorelease];
			    if (![self isPrintable:bvals[i]->bv_val]) childString=[childData description];
			    id newAttr=[personAttributes objectForKey:[NSString stringWithUTF8String:resultstring]];
			    if (newAttr) {
				if ([newAttr isKindOfClass:[NSArray class]]) {
				    NSMutableArray *tempArray=[NSMutableArray arrayWithObject:childString];
				    [tempArray addObjectsFromArray:newAttr];
				    [personAttributes setObject:tempArray forKey:[NSString stringWithUTF8String:resultstring]];
				}
				else {
				    NSArray *tempArray=[NSArray arrayWithObjects:childString,newAttr,nil];
				    [personAttributes setObject:tempArray forKey:[NSString stringWithUTF8String:resultstring]];
				}
			    }
			    else {
				[personAttributes setObject:childString forKey:[NSString stringWithUTF8String:resultstring]];
			    }
			}
			[person setObject:personAttributes forKey:@"attributes"];
			#ifdef DEBUG
			    NSLog(@"person is %@",[person description]);
			#endif
		    }
		    ldap_memfree( resultstring );
		}
		if ( berptr != NULL ) ber_free(berptr, 0);
		ldap_memfree(dn);
		[notifyObject performSelector:notifySelector withObject:person];
	    }


	ldap_msgfree(result);
	ldap_msgfree(resultmessage);    

	    break;


	case -1:
	    bound=NO;
	    NSLog(@"Error code:%x (%s)",res,ldap_err2string(res));
	    [self setLastError:[NSString stringWithFormat:@"%s",ldap_err2string(res)]];
	    break;
	case LDAP_RES_SEARCH_RESULT:
	    [[NSNotificationCenter defaultCenter] postNotificationName:@"SearchCompleted" object:self userInfo:[NSDictionary dictionaryWithObject:[NSNumber numberWithInt:msgid] forKey:@"msgid"]];
	    [searchTimers removeObjectForKey:[NSString stringWithFormat:@"%i",msgid]];
	    [timer invalidate];
	    break;
    }
}
-(void)cancelSearchWithMsgID:(int)msgid{  
    NSTimer *timer=[searchTimers objectForKey:[NSString stringWithFormat:@"%i",msgid]];
    [timer invalidate];
    [searchTimers removeObjectForKey:[NSString stringWithFormat:@"%i",msgid]];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"SearchCompleted" object:self userInfo:[NSDictionary dictionaryWithObject:[NSNumber numberWithInt:msgid] forKey:@"msgid"]];
    if(ldap_abandon_ext( ld, msgid, nil,nil ) != LDAP_SUCCESS )
	ldap_perror( ld, "ldap_abandon" );  
    
}
#pragma mark Binding Methods
-(BOOL)asyncBindWithSender:(id)sender completedSelector:(SEL)aSelector{
//-(LDAP *)bind{
    int rc;
    if (!useSSL) ld=ldap_init([hostURL UTF8String], port); 
    else {
	rc=ldap_initialize( &ld, [[NSString stringWithFormat:@"ldaps://%@:%i",hostURL,port] UTF8String]);
	if( rc != LDAP_SUCCESS ) {
	    fprintf( stderr, "Could not create LDAP session handle (%d): %s\n", rc, ldap_err2string(rc) );
	   // return nil;
	}
    }
    int version=3;
    struct berval cred ={ 0, NULL };
    int err,msgid;
    static int timelimit;
    timelimit=TIMELIMIT;
    ldap_set_option( ld, LDAP_OPT_PROTOCOL_VERSION, &version );
    if (ldap_set_option( ld, LDAP_OPT_TIMELIMIT, (void *) &timelimit ) != LDAP_OPT_SUCCESS )
    {
	fprintf( stderr, "Could not set LDAP_OPT_TIMELIMIT %d\n", timelimit );
	exit( EXIT_FAILURE );
    }
    
    if (authType==AUTH_MD5) {
	void *defaults;
	
	char *sasl_realm = NULL;
	char *sasl_authc_id = NULL;
	char *sasl_authz_id = NULL;
	char *sasl_mech = NULL;
	char *sasl_secprops = NULL;
	struct berval passwd = { 0, NULL };
	unsigned sasl_flags = LDAP_SASL_AUTOMATIC;
	
	cred.bv_val=malloc (sizeof(char)*[bindUsername length]);
	strcpy(cred.bv_val,[bindUsername UTF8String]);
	//cred.bv_val = (char *) [bindUsername UTF8String];
	cred.bv_len = strlen(cred.bv_val);
	
	passwd.bv_val=malloc (sizeof(char)*[bindPassword length]);
	strcpy(passwd.bv_val,[bindPassword UTF8String]);
	//passwd.bv_val = (char *) [bindPassword UTF8String];
	
	sasl_authc_id=malloc (sizeof(char)*[saslUsername length]);
	strcpy(sasl_authc_id,[saslUsername UTF8String]);
	//sasl_authc_id= (char *) [saslUsername UTF8String];
	
	passwd.bv_len = strlen( passwd.bv_val );
	if( sasl_secprops != NULL ) {
	    err = ldap_set_option( ld, LDAP_OPT_X_SASL_SECPROPS,
				   (void *) sasl_secprops );
	    
	    if( err != LDAP_OPT_SUCCESS ) {
		NSLogv(@"Could not set LDAP_OPT_X_SASL_SECPROPS: %s",sasl_secprops );
		exit( EXIT_FAILURE );
	    }
	}
	
	//msgid=ldap_simple_bind(ld, [bindUsername UTF8String], [bindPassword UTF8String]);
	/*defaults = lutil_sasl_defaults( ld,
					sasl_mech,
					sasl_realm,
					sasl_authc_id,
					passwd.bv_val,
					sasl_authz_id );
*/

	//ldap_sasl_bind(ld,[bindUsername UTF8String],sasl_mech,&cred,(void *)0,(void *)0q,&msgid);

		
	
	/*err = ldap_sasl_interactive_bind_s( ld,cred.bv_val,
					    sasl_mech, NULL, NULL,
					    sasl_flags, lutil_sasl_interact, defaults );
	*/
	//lutil_sasl_freedefs( defaults );
	strlcpy(sasl_authc_id,"",sizeof(passwd.bv_val));
	free(sasl_authc_id);
	strlcpy(passwd.bv_val,"",sizeof(passwd.bv_val));
	free(passwd.bv_val);
	free(cred.bv_val);
	
	/*if( err != LDAP_SUCCESS ) {
	    [self setLastError:[NSString stringWithUTF8String:ldap_err2string(err)]];
	    ldap_perror( ld, "ldap_sasl_interactive_bind_s" );
	    NSLog(@"Error code:%x (%s)",err,ldap_err2string(err));
	    
	    //exit( EXIT_FAILURE );
	*/
	NSDictionary *userInfoDict=[NSDictionary dictionaryWithObjectsAndKeys:sender,@"sender",[NSValue valueWithPointer:aSelector],@"selector",[NSNumber numberWithInt:msgid],@"msgid",nil];
	bindTimer=[NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(checkBindStatus:) userInfo:userInfoDict repeats:YES];
	//}
	
    } else
    { 
	
	int msgid;
	//err=ldap_simple_bind_s(ld, [bindUsername UTF8String], [bindPassword UTF8String]);
	
	struct timeval tv;
	tv.tv_sec=3;
	tv.tv_usec=0;
	
	ldap_set_option(ld, LDAP_OPT_NETWORK_TIMEOUT, &tv);

	msgid=ldap_simple_bind(ld, [bindUsername UTF8String], [bindPassword UTF8String]);
	if (msgid==-1) {
	   // NSRunAlertPanel(@"Error!",@"Network related error.  Make sure the dns name is correct, and that the server is accepting connections.", @"OK", nil,nil);
	    NSLog(@"network error");
	    [self setLastError:[NSString stringWithFormat:@"%s",ldap_err2string(err)]];
	    return NO;
	}
	else {
	    NSDictionary *userInfoDict=[NSDictionary dictionaryWithObjectsAndKeys:sender,@"sender",[NSValue valueWithPointer:aSelector],@"selector",[NSNumber numberWithInt:msgid],@"msgid",nil];
	    bindTimer=[NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(checkBindStatus:) userInfo:userInfoDict repeats:YES];
	}
    }
    return YES;
}
-(void)cancelBind{
    [bindTimer invalidate];
    [self unbind];
}
-(void)simpleBind{
    int rc;
    if (!useSSL) ld=ldap_init([hostURL UTF8String], port); 
    else {
	rc=ldap_initialize( &ld, [[NSString stringWithFormat:@"ldaps://%@:%i",hostURL,port] UTF8String]);
	if( rc != LDAP_SUCCESS ) {
	    fprintf( stderr, "Could not create LDAP session handle (%d): %s\n", rc, ldap_err2string(rc) );
	    // return nil;
	}
    }
    int version=3;
    int err;
    static int timelimit;
    timelimit=TIMELIMIT;
    ldap_set_option( ld, LDAP_OPT_PROTOCOL_VERSION, &version );
    
    if (ldap_set_option( ld, LDAP_OPT_TIMELIMIT, (void *) &timelimit ) != LDAP_OPT_SUCCESS )
    {
	fprintf( stderr, "Could not set LDAP_OPT_TIMELIMIT %d\n", timelimit );
	exit( EXIT_FAILURE );
    }
    
    err=ldap_simple_bind_s(ld, [bindUsername UTF8String], [bindPassword UTF8String]);

    if (err!=0) {
	bound=NO;
	NSLog(@"Error code:%x (%s)",err,ldap_err2string(err));
	[self setLastError:[NSString stringWithFormat:@"%s",ldap_err2string(err)]];
    } else bound=YES;
    
}
-(void)checkBindStatus:(NSTimer *)sender{
    int err;
    LDAPMessage *result;
    struct timeval timeout;
    timeout.tv_sec=0;
    timeout.tv_usec=0;
    
    err=ldap_result( ld, [[[sender userInfo] objectForKey:@"msgid"] intValue], 0, &timeout, &result);
    if (err==0) {
	NSLog(@"not bound yet error=%i",err);
    }
    else if (err==-1) {
	bound=NO;
	NSLog(@"Error code:%x (%s)",err,ldap_err2string(err));
	[self setLastError:[NSString stringWithFormat:@"%s",ldap_err2string(err)]];
    } else 
	if (err==LDAP_RES_BIND) {
	    bound=YES;

	    id notifyObject=[[sender userInfo] objectForKey:@"sender"];

	    SEL notifySelector=[[[sender userInfo] objectForKey:@"selector"] pointerValue];
		[sender invalidate];	    
	    [notifyObject performSelector:notifySelector withObject:self];

	}
}
-(BOOL)unbind{
    BOOL returnVal=NO;
    if (bound) ldap_unbind(ld);
    bound=NO;
    return returnVal;
}
#pragma mark Modify Methods
-(BOOL)deleteEntry:(NSString *)dn
{
    int result;
    BOOL returnVal=NO;
    result=ldap_delete_ext_s(ld, (char *)[dn UTF8String],NULL,NULL);
    if (result==0) returnVal=YES;
    NSLog(@"Error code:%x (%s)",result,ldap_err2string(result));
    [self setLastError:[NSString stringWithUTF8String:ldap_err2string(result)]];
    return returnVal;
}

-(BOOL)modifyEntry:(NSString *)dn withAttributes:(NSDictionary *)person{
    int result,i;
    BOOL returnVal=NO;
    LDAPMod **mods;
    //struct berval photo_berval;
    
    mods = ( LDAPMod ** ) malloc(( [person count] + 1 ) * sizeof( LDAPMod * )); 
    if ( mods == NULL ) { 
	fprintf( stderr, "Cannot allocate memory for mods array\n" ); 
    }
    NSArray *valueArray;
    NSEnumerator *keyEnumerator=[person keyEnumerator];
    NSString *key;
    NSString *currentValue;
    int j=0;
    i=0;
  //  LDAPMod *ldapmod;
    char **values;
    while (key=[keyEnumerator nextObject]) {
	if (( mods [ i ] = ( LDAPMod * ) malloc( sizeof( LDAPMod ))) == NULL) fprintf( stderr, "Cannot allocate memory for mods element\n" ); 
	valueArray=[person objectForKey:key];
	mods[i]->mod_op=LDAP_MOD_REPLACE;
	mods[i]->mod_type=( char *)[key UTF8String];
	//char *values[[valueArray count]];

	//NSLog(@"[valueArray count] is %i",[valueArray count]);
	values=(char **)malloc([valueArray count] * sizeof(char *) + 1);
	for (j=0;j<[valueArray count];j++) {
	    currentValue=[valueArray objectAtIndex:j] ;

	    //NSLog(@"pointer to values is %p",values);
	    if (![currentValue isKindOfClass:[NSData class]]) {
		//NSLog(@"mods[%i]->mod_type is %s",i,mods[i]->mod_type);
		if ([currentValue isEqualToString:@""]) values[j]=NULL;
		else {
		    values[j]=( char *)[currentValue UTF8String];
		    //NSLog(@"values[%i]=%s",j,values[j]);
		}
	    }
	}
	//NSLog(@"setting values[%i] to NULL",j);
	values[j]=NULL;
	mods[i]->mod_values=&values[0];
	i++;
    }
   // NSLog(@"setting     mods[%i]=NULL", i);
    mods[i]=NULL;
    result=ldap_modify_ext_s( ld, [dn UTF8String], mods, NULL, NULL );
    if (result==0) {
	NSLog(@"successfully modified");
	returnVal=YES;
    }
    else {
//	ldap_mods_free(mods,1); //here
	NSLog(@"error modifying");
    }
    return returnVal;
}
-(BOOL)addEntry:(NSString *)dn withAttributes:(NSDictionary *)person{
    
    struct berval photo_berval;
    int result,i,j,k,l;
    BOOL returnVal=NO;
    LDAPMod **mods;
    mods = ( LDAPMod ** ) malloc( ([person count]+1)  * sizeof( LDAPMod * )); 
    if ( mods == NULL ) { 
	fprintf( stderr, "Cannot allocate memory for mods array\n" ); 
    }
    for ( i = 0; i < [person count]; i++ ) { 
	if (( mods[ i ] = ( LDAPMod * ) malloc( sizeof( LDAPMod ))) == NULL) {
	    fprintf( stderr, "Cannot allocate memory for mods element\n" ); 
	} 
    }
    char **values;
    
    values=(char **)malloc(sizeof(char *)*[person count]*3);
    //[[person count]][2];
    NSArray *keys=[person allKeys];
    j=0;
    k=0;
    l=0;
    for (i=0;i<[person count];i++) {
	id value=[person objectForKey:[keys objectAtIndex:i]];
	if (![value isKindOfClass:[NSData class]]) {
	    
	    //if(![[person objectForKey:[keys objectAtIndex:i]] isEqualToString:@""]){
	    mods[l]->mod_op=LDAP_MOD_REPLACE;
	    mods[l]->mod_type=( char *)[[keys objectAtIndex:i] UTF8String];
	    if ([[person objectForKey:[keys objectAtIndex:i]] isEqualToString:@""]) {
		values[j]=NULL;
		k=j;
	    }
	    else {
		values[j]=( char *)[[person objectForKey:[keys objectAtIndex:i]] UTF8String];
		values[j+1]=NULL;
		//NSLog(@"%i %s",l,values[j]);
		k=j+1;
	    }
	    
	    
	    }
	else {
	    
	    mods[l]->mod_op=LDAP_MOD_REPLACE|LDAP_MOD_BVALUES;
	    mods[l]->mod_type=( char *)[[keys objectAtIndex:i] UTF8String];
	    NSData *photoData=[person objectForKey:[keys objectAtIndex:i]];
	    photo_berval.bv_len = [photoData length];
	    photo_berval.bv_val =  (void *)[[person objectForKey:[keys objectAtIndex:i]] bytes];
	    
	    values[j]=(char *)&photo_berval;
	    values[j+1]=NULL;
	    k=j+1;
	    
	}
	mods[l]->mod_values=&values[j];
	j=k+1;
	l++;
	}
    mods[l]=NULL;
    result=ldap_add_s( ld, [dn UTF8String], mods);
    
    if (result==0) returnVal=YES;
    else {
	//ldap_mods_free(mods,1);
	NSLog(@"%@:%s",dn,ldap_err2string( result ));
    }
    free (values);
    for ( i = 0; i < [person count]; i++ ) { 
	free( mods[ i ]);
    }
    
    free (mods);
    
    //free(values[]);
    return returnVal;
    }
   
-(BOOL)rename:(NSString *)dn toNewDN:(NSString *)newDN{
    NSLog(@"gtring to rename %@ to %@",dn,newDN);
    int result;
    BOOL returnVal=NO;
    //  if (err) NSLog(@"%s",ldap_err2string( err ));
    result=ldap_modrdn2_s(ld, (char *)[dn UTF8String],[[NSString stringWithFormat:@"cn=%@",newDN] UTF8String],TRUE);
    if (result==0) returnVal=YES;
    return returnVal;
    
    
}
#pragma mark Printing Methods
-(BOOL)isPrintable:(char *)val{
    int i;
    if( isgraph(  val[0] ) && val[0] != ':'){
	for ( i = 0; val[i]; i++ ) {
	    if ( !isascii( val[i] ) && !isprint( val[i] ) ) {
		return NO;
	    }
	}
	return YES;
    }
    return NO;
}
#pragma mark Accessor Methods
-(void)setHostURL:(NSString *)val{
    [val retain];
    [hostURL release];
    hostURL=val;
}
-(void)setBaseDN:(NSString *)val{
    [val retain];
    [baseDN release];
    baseDN=val;
}
-(void)setBindUsername:(NSString *)val{
    [val retain];
    [bindUsername release];
    bindUsername=val;
}
-(void)setBindPassword:(NSString *)val{
    [val retain];
    [bindPassword release];
    bindPassword=val;
}
-(void)setSASLUsername:(NSString *)val{
    [val retain];
    [saslUsername release];
    saslUsername=val;
}
-(void)setLastError:(NSString *)val{
    [val retain];
    [lastError release];
    lastError=val;
}
-(NSString *)lastError{
    return lastError;
}

-(BOOL)bound{
    return bound;
}
#pragma mark Memory Management
-(void)dealloc{
    [self unbind];
    [searchTimers release];
    [super dealloc];
}
@end
