//
//  MyDocument.m
//  LDAPManager
//
//  Created by Timothy Perfitt on Fri Apr 16 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

#import "MyDocument.h"

@implementation MyDocument
- (NSString *)windowNibName
{
    return @"MyDocument";
}

#pragma mark initializers
-(void)awakeFromNib{
    toolbarItems=[[NSMutableDictionary alloc] init];
    [self addToolbarItemToDictionary:toolbarItems label:@"New" paletteLabel:@"New" target:self action:@selector(new:) toolTip:@"Add new entry or attribute" identifier:@"new" image:[NSImage imageNamed:@"newFolder.tiff"]];
    [self addToolbarItemToDictionary:toolbarItems label:@"Delete Item" paletteLabel:@"Delete Item" target:self action:@selector(deleteEntry:) toolTip:@"Delete entry at current position" identifier:@"deleteEntry" image:[NSImage imageNamed:@"deleteIcon.tiff"]];
    [self addToolbarItemToDictionary:toolbarItems label:@"Duplicate Item" paletteLabel:@"Duplicate Item" target:self action:@selector(duplicateEntry:) toolTip:@"Duplicate Item at current position" identifier:@"duplicateEntry" image:[NSImage imageNamed:@"duplicateFolder.tiff"]];
    [self addToolbarItemToDictionary:toolbarItems label:@"Refresh" paletteLabel:@"Refresh" target:self action:@selector(refresh:) toolTip:@"Refresh" identifier:@"refresh" image:[NSImage imageNamed:@"Refresh.tiff"]];
    
    NSToolbar *tb=[[[NSToolbar alloc] initWithIdentifier:@"mainToolbar"] autorelease];
    [tb setDelegate:self];
    [tb setAllowsUserCustomization:YES];
    [tb setAutosavesConfiguration: YES];
    [mainWindow setToolbar:tb];
    
    
    [connectButton setEnabled:NO];
    [fetchButton setEnabled:NO];
    [typeDesc setTextColor:[NSColor lightGrayColor]];
    [usernameDesc setTextColor:[NSColor lightGrayColor]];
    [passwordDesc setTextColor:[NSColor lightGrayColor]];
   
    ldap=nil;
    runningSearches=[[NSMutableArray alloc] init];
}

- (void)windowControllerDidLoadNib:(NSWindowController *) aController
{
    [super windowControllerDidLoadNib:aController];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(selectionShouldChange:) name:@"MatrixChanged" object:nil];
    [NSApp beginSheet:connectWindow modalForWindow:mainWindow modalDelegate:self didEndSelector:@selector(sheetDidEnd:returnCode:contextInfo:) contextInfo:nil];

}

#pragma mark actions from buttons
- (IBAction)commit:(id)sender
{
    if ([mainWindow isDocumentEdited]){ 
	if ([self commit])[mainWindow setDocumentEdited:NO];
    }
}

- (IBAction)deleteAttribute:(id)sender
{
}

- (IBAction)deleteEntry:(id)sender
{
    
    if ([attributeOutlineView selectedRow]!=-1){
        NSLog(@"delete attribute %i",[attributeOutlineView selectedRow]);
    }
    else {
	
        NSString *deletedn=[entryBrowserView currentDN];
        int rc=NSRunCriticalAlertPanel(@"Delete Entry", [NSString stringWithFormat:@"Are you sure you want to delete the entry at %@?",deletedn], @"OK", @"Cancel", nil);
        if(rc==1) {
	    if(![ldap deleteEntry:deletedn]) NSRunAlertPanel(@"Error!", [NSString stringWithFormat:@"Could not delete entry!  %@",[ldap lastError]], @"OK",nil, nil);
	    else {
		DataSource *ds=[entryBrowserView delegate];
		[ds deleteEntryAtPath:[entryBrowserView pathArray]];
	    }
        } 
    }
    
}
- (IBAction)refresh:(id)sender{
    DataSource *ds=[entryBrowserView delegate];
    [ds refresh];
}



- (IBAction)insertAttribute:(id)sender
{
}


- (void)printShowingPrintPanel:(BOOL)flag{
    [[mainWindow firstResponder] print:self];
}
- (IBAction)new:(id)sender
{
    
}

- (IBAction)duplicateEntry:(id)sender
{
    
}

- (IBAction)revert:(id)sender
{
}

- (IBAction)authMethodChanged:(id)sender{
    
    switch ([[sender selectedItem] tag]) {
	case 1: [usernameDesc setStringValue:@"Distinguished Name:"];
	    break;
	case 2: [usernameDesc setStringValue:@"SASL Username:"];
	    break;
	default: 
	    break;
    }
    [self checkKeychain];
}
- (IBAction)useAuthentication:(id)sender
{
    if([useAuthenticationCheckBox state] == NSOnState) {
	[passwordTextfield setEnabled:YES];
	[userNameTextfield setEnabled:YES];
	[authPopUpButton setEnabled:YES];
	[keychainCheckBox setEnabled:YES];
	[typeDesc setTextColor:[NSColor blackColor]];
	[usernameDesc setTextColor:[NSColor blackColor]];
	[passwordDesc setTextColor:[NSColor blackColor]];
    } else {
	[passwordTextfield setEnabled:NO];
	[userNameTextfield setEnabled:NO];
	[authPopUpButton setEnabled:NO];
	[keychainCheckBox setEnabled:NO];
	[typeDesc setTextColor:[NSColor lightGrayColor]];
	[usernameDesc setTextColor:[NSColor lightGrayColor]];
	[passwordDesc setTextColor:[NSColor lightGrayColor]];
    }
    [self checkKeychain];
}
- (IBAction)encrypt:(id)sender
{
    if([sslCheckBox state] == NSOnState) {
	if([customPortCheckBox state] == NSOffState)
	    [portTextField setStringValue:@"636"];
    } else {
	if([customPortCheckBox state] == NSOffState)
	    [portTextField setStringValue:@"389"];
    } // end if
    [self checkKeychain];
}
- (IBAction)customPort:(id)sender
{
    if([customPortCheckBox state] == NSOnState) {
	[portTextField setEnabled:YES];
    } else {
	[portTextField setEnabled:NO];
    }
    [self checkKeychain];
}


- (IBAction)fetchDNButton:(id)sender{
    [fetchButton setEnabled:NO];
    ldap=[[LDAPQuery alloc] initWithHostURL:[hostTextField stringValue] port:[portTextField intValue] authType:AUTH_ANONYMOUS bindUsername:nil bindPassword:nil dn:@"" saslUsername:nil overSSL:[sslCheckBox state]];

    if(![ldap asyncBindWithSender:self completedSelector:@selector(getFetchDNBindStatus:)]) {
	NSRunAlertPanel(@"Bind Error!", @"Please check the DNS name and network connection and try again.", @"OK", nil,nil);
    }
}
-(void)getFetchDNBindStatus:(id)sender{
    if ([ldap bound]) {
	
	[ldap searchWithFilter:@"(objectclass=*)" forAttributes:[NSArray arrayWithObject:@"namingContexts"] scope:LDAP_SCOPE_BASE searchDN:@"" target:self selector:@selector(getDNResult:)];
	
    }
    else {
	NSRunAlertPanel(@"Error!", [ldap lastError], @"OK", nil,nil);
	[fetchButton setEnabled:YES];
    }
    
}
-(void)getDNResult:(NSMutableDictionary *)newChildren{
    
    [baseDNTextField setStringValue:[[newChildren objectForKey:@"attributes"] objectForKey:@"namingContexts"]];
    [self checkKeychain];
    [fetchButton setEnabled:YES];
}
-(IBAction)cancelConnecting:(id)sender{
    if (isConnecting) {
	isConnecting=NO;
	[ldap cancelBind];
	[busyGear stopAnimation:self];
	[connectButton setEnabled:YES];
	[busyGear setHidden:YES];

    }
    else {
	//[connectWindow orderOut: self];
	[NSApp endSheet:connectWindow];

	[mainWindow close];
    }
}
- (IBAction)connect:(id)sender{
    [connectButton setEnabled:NO];
    [busyGear setHidden:NO];
    isConnecting=YES;
    [busyGear startAnimation:self];
    if(([useAuthenticationCheckBox state] == NSOnState) && ( ([[passwordTextfield stringValue] length] == 0) || ([[userNameTextfield stringValue] length]==0) )) {
	NSRunAlertPanel(@"Invalid Entries!", @"Please make sure that you have entered in a username and password if you are using authenication.", @"OK", nil, nil);
	return;
    }
    
    NSDictionary *dict = [self prefDataFromInterface];    
    port = [[dict objectForKey:@"port"] intValue];
    useSSL= [[dict objectForKey:@"useSSL"] boolValue];
    authType= [[dict objectForKey:@"authType"] intValue];
    addToKeychain = [[dict objectForKey:@"addToKeychain"] boolValue];
    [self setBaseDN:[dict objectForKey:@"baseDN"]];
    [self setHost:[dict objectForKey:@"host"]];
    [self setPassword:[dict objectForKey:@"password"]];
    [self setUserName:[dict objectForKey:@"userName"]];
    //[self updateChangeCount:NSChangeDone];
    //dirty=YES;
    
    [self connect];

    
}
-(BOOL)connect{
    switch (authType) {
	case 1:ldap=[[LDAPQuery alloc] initWithHostURL:host port:port authType:AUTH_SIMPLE bindUsername:userName  bindPassword:password dn:baseDN saslUsername:nil overSSL:useSSL];
	    [ldap asyncBindWithSender:self completedSelector:@selector(connectionComplete:)];
	    break;
	case 2:
	    
	    ldap=[[LDAPQuery alloc] initWithHostURL:host port:port authType:AUTH_MD5 bindUsername:@"" bindPassword:password dn:baseDN saslUsername:userName overSSL:useSSL];
	    if(![ldap asyncBindWithSender:self completedSelector:@selector(connectionComplete:)]) {
		[connectButton setEnabled:YES];
		[busyGear stopAnimation:self];
		[busyGear setHidden:YES];
		return NO;
	    }
	    break;
	default:
	    ldap=[[LDAPQuery alloc] initWithHostURL:host port:port authType:AUTH_ANONYMOUS bindUsername:nil bindPassword:nil dn:baseDN saslUsername:nil overSSL:useSSL];
	    if(![ldap asyncBindWithSender:self completedSelector:@selector(connectionComplete:)]) {
		[connectButton setEnabled:YES];
		[busyGear stopAnimation:self];
		[busyGear setHidden:YES];
		NSRunAlertPanel(@"Bind Error!", @"Please check the DNS name and network connection and try again.", @"OK", nil,nil);
		return NO;
	    }
	    break;
    } // end switch
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(searchRunning:) name:@"SearchStarted" object:ldap];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(searchCompleted:) name:@"SearchCompleted" object:ldap];
    return YES;
}
#pragma mark Connecting Methods
-(void)connectionComplete:(id)sender{
	[busyGear stopAnimation:self];
	[connectButton setEnabled:YES];
	[busyGear setHidden:YES];

    	isConnecting=NO;
    if ([ldap bound]){
	[[entryBrowserView delegate] setLDAP:ldap];
	[[entryBrowserView delegate] setBaseDN:baseDN];
	[entryBrowserView loadColumnZero];
	[mainWindow setTitle:[NSString stringWithFormat:@"%@%@:%@/%@", (useSSL == YES ? @"ldaps://" : @"ldap://"), host, [[NSNumber numberWithInt:port] stringValue], baseDN]];
	[entryBrowserView selectRow:0 inColumn:0];
	[entryBrowserView doClick:nil];
	// add or update keychain item
	//NSLog(@"Add to keychain = %@   AuthType = %d", (addToKeychain == YES ? @"YES" : @"NO"), authType);
	if((authType == 1) && (addToKeychain == YES)) [KeychainAccess storePasswordInKeychainForServer:host account:userName password:password path:baseDN port:[NSString stringWithFormat:@"%d",port] useSSL:useSSL useSASL:NO];
	if((authType == 2) && (addToKeychain == YES)) [KeychainAccess storePasswordInKeychainForServer:host account:userName password:password path:baseDN port:[NSString stringWithFormat:@"%d",port] useSSL:useSSL useSASL:YES];
	//[connectWindow orderOut: self];
	[NSApp endSheet:connectWindow];

    } else {
	NSRunAlertPanel(@"Error!", [ldap lastError], @"OK", nil, nil);
    } // end if
    
}
- (void)didEndSheet:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    NSLog(@"didEndSheet");
    [sheet orderOut:self];
}
#pragma mark keychain
- (void)checkKeychain
{

    if(([useAuthenticationCheckBox state] == NSOnState) && ([[userNameTextfield stringValue] length] != 0) && ([[hostTextField stringValue] length] !=0)) {
	NSString *temp = [KeychainAccess getPasswordInKeychainWithServer:[hostTextField stringValue] account:[userNameTextfield stringValue] path:[baseDNTextField stringValue] port:[portTextField stringValue] useSSL:([sslCheckBox state] == NSOnState ? YES : NO) useSASL:([[authPopUpButton selectedItem] tag] == 1 ? NO : YES)];
	[passwordTextfield setStringValue:temp];
    }
}
#pragma mark Spinner
- (void)searchRunning:(NSNotification *)aNotification{

    [spinner setHidden:NO];
    [spinner startAnimation:self];
    NSNumber *msgid=[[aNotification userInfo] objectForKey:@"msgid"];
    [runningSearches addObject:msgid];
    
}
- (void)searchCompleted:(NSNotification *)aNotification{
    int i;
    for (i=0;i<[runningSearches count];i++){
	NSNumber *msgid=[[aNotification userInfo] objectForKey:@"msgid"];
	if ([[runningSearches objectAtIndex:i] intValue]==[msgid intValue]) {
	    [runningSearches removeObjectAtIndex:i];
	}
    }
    if ([runningSearches count]==0) {
	[spinner stopAnimation:self];
	[spinner setHidden:YES];
    }
    
    
}
#pragma mark keyboard input
- (void)controlTextDidChange:(NSNotification *)aNotification
{
    if([[hostTextField stringValue] isEqual:@""]) {
	[connectButton setEnabled:NO];
	[fetchButton setEnabled:NO];
    } else {
	if (!isConnecting) [connectButton setEnabled:YES];
	[fetchButton setEnabled:YES];
    }
    [self checkKeychain];
}

#pragma mark Toolbar
-addToolbarItemToDictionary:(NSMutableDictionary *)dict label:(NSString *)label paletteLabel:(NSString *)pl target:(id)target action:(SEL)action toolTip:(NSString *)toolTip identifier:(NSString *)identifier image:(NSImage *)image{
    
    NSToolbarItem *tbi=[[[NSToolbarItem alloc] initWithItemIdentifier:identifier] autorelease];
    [tbi setLabel:label];
    [tbi setPaletteLabel:pl];
    [tbi setTarget:target];
    [tbi setToolTip:toolTip];
    [tbi setAction:action];
    [tbi performSelector:@selector(setImage:) withObject:image];
    [dict setObject:tbi forKey:identifier];
    
    return dict;
}
- (NSArray *)toolbarAllowedItemIdentifiers:(NSToolbar *)toolbar{
    return [NSArray arrayWithObjects: 
        @"refresh",NSToolbarPrintItemIdentifier,
	NSToolbarCustomizeToolbarItemIdentifier,
        NSToolbarFlexibleSpaceItemIdentifier,
        NSToolbarSpaceItemIdentifier, nil];
    
}

- (NSToolbarItem *)toolbar:(NSToolbar *)toolbar itemForItemIdentifier:(NSString *)itemIdentifier willBeInsertedIntoToolbar:(BOOL)flag{
    NSToolbarItem *newItem=[[[toolbarItems objectForKey:itemIdentifier] copy] autorelease];
    return newItem;
}

- (NSArray *)toolbarDefaultItemIdentifiers:(NSToolbar *)toolbar{
    
    return [NSArray arrayWithObjects: 
        @"refresh",NSToolbarPrintItemIdentifier,
	NSToolbarCustomizeToolbarItemIdentifier,
        NSToolbarFlexibleSpaceItemIdentifier,
        NSToolbarSpaceItemIdentifier, nil];
}


#pragma mark delegate methods
- (BOOL)windowShouldClose:(id)sender{
    if ([sender isDocumentEdited]) {
	NSBeginCriticalAlertSheet(@"Unsaved Data!", @"Commit", @"Revert", @"Cancel", mainWindow, self, @selector(sheetDidEnd: returnCode: contextInfo:), NULL, NULL, @"You modified some attributes.  Do you want to commit these changes, revert back to the version before you edited them, or cancel to return back to editing the attibutes?");
	return NO;
    }
    else return YES;

}

#pragma mark end sheet
-(void)sheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo{
    switch (returnCode) {
	case  0:  //revert
	    [mainWindow setDocumentEdited:NO];
	    [mainWindow close];
	    break;
	case -1:  //cancel
	    break;
	case 1: // save
	    if([self commit]) {
		[mainWindow setDocumentEdited:NO];
		[mainWindow close];
	    } 
	    break;
    }
    [sheet orderOut:self];
}

-(void)selectionShouldChange:(NSNotification *)sender{
    if ([mainWindow isDocumentEdited]) {
	NSBeginCriticalAlertSheet(@"Unsaved Data!", @"Commit", @"Revert", @"Cancel", mainWindow, self, @selector(selectionChangedSheetDidEnd: returnCode: contextInfo:), NULL, NULL, @"You modified some attributes.  Do you want to commit these changes, revert back to the version before you edited them, or cancel to return back to editing the attibutes?");
	[[sender object] setDirty:YES];
    }
    else [[sender object] setDirty:NO];
    
    
}
-(void)selectionChangedSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo{
    switch (returnCode) {
	case  0:  //revert
	    [mainWindow setDocumentEdited:NO];
	    [self refreshAttributes:self];
	    break;
	case -1:  //cancel
	    break;
	case 1: // save
	    if([self commit]) {
		[mainWindow setDocumentEdited:NO];
	    } 
		break;
    }

     [sheet orderOut:self];
}

#pragma mark set interface and give info from interface
-(void)setInterfaceUseSSL:(BOOL)ssl authType:(int)at host:(NSString *)thost port:(int)tport baseDN:(NSString *)dn addToKeychain:(BOOL)keychain username:(NSString *)username{
    if(ssl == NSOnState)
	[sslCheckBox setState:NSOnState];
    else
	[sslCheckBox setState:NSOffState];

    [hostTextField setStringValue:thost];
    [portTextField setIntValue:tport];
    [baseDNTextField setStringValue:dn];
    [self controlTextDidChange:nil];

    if(keychain == NSOnState)
	[keychainCheckBox setState:NSOnState];
    else
	[keychainCheckBox setState:NSOffState];
    
    [userNameTextfield setStringValue:username];
    [passwordTextfield setStringValue:@""];
    if (!isConnecting) [connectButton setEnabled:YES];
    [fetchButton setEnabled:YES];
    
    switch(at) {
	case 0: [userNameTextfield setStringValue:@""];
	    [useAuthenticationCheckBox setState:NSOffState];
	    break;
	case 1: [useAuthenticationCheckBox setState:NSOnState];
	    [self useAuthentication:nil];
	    [authPopUpButton selectItemAtIndex:0];
	    break;
	case 2: [useAuthenticationCheckBox setState:NSOnState];
	    [self useAuthentication:nil];
	    [authPopUpButton selectItemAtIndex:1];
	    break;
    }
    [self checkKeychain];
    
    
}

-(NSDictionary *)prefDataFromInterface{
    NSDictionary *temp;
    temp = [NSDictionary dictionaryWithObjectsAndKeys:[baseDNTextField stringValue], @"baseDN",
	[hostTextField stringValue],@"host",
	[passwordTextfield stringValue],@"password",
	[portTextField stringValue],@"port",
	[NSNumber numberWithInt:[sslCheckBox state]],@"useSSL",
	[userNameTextfield stringValue],@"userName",
	[NSNumber numberWithInt:([useAuthenticationCheckBox state] == NSOnState) ? [[authPopUpButton selectedItem] tag] : 0	],@"authType",
	[NSNumber numberWithInt:[keychainCheckBox state]], @"addToKeychain",
	nil];
    return  temp;
    
}

#pragma mark commit
-(BOOL)commit{
    /*[mainWindow endEditingFor:nil];
    DataSource *ds=[entryBrowserView delegate];
    LDAPAttribute *lda=[ds currentAttributes];
    [lda childrenAttributes];

    ///NSLog(@"current attib ues is %@",[lda childrenAttributes]);

    if([ldap modifyEntry:[entryBrowserView currentDN] withAttributes:[lda childrenAttributes]]) return YES;
    else {
	NSRunAlertPanel(@"Error!", @"Could not commit entry!  Make sure you have write privleges to this entry.", @"OK",nil, nil);
	return NO;
	
    }
    */
}
#pragma assessor and set methods
-(void)setBaseDN:(NSString *)val{
    [val retain];
    [baseDN release];
    baseDN=val;
        
}
-(void)setHost:(NSString *)val{
    [val retain];
    [host release];
    host=val;
}
-(void)setPassword:(NSString *)val{
    [val retain];
    [password release];
    password=val;
}

-(void)setUserName:(NSString *)val{
    [val retain];
    [userName release];
    userName=val;
}
/*- (IBAction)connectionWindow:(id)sender{
    NSLog(@"connectionWindow");
    [NSApp beginSheet:connectWindow modalForWindow:mainWindow modalDelegate:self didEndSelector:@selector(didEndSheet:returnCode:contextInfo:) contextInfo:nil];
        NSLog(@"connectionWindow");
}
*/
-(id)ldap{
    return ldap;
}
#pragma mark dealloc

-(void)dealloc{
    [ldap release];
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [runningSearches release];
    [super dealloc];
}

@end

